
/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.11B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#ifndef __QNXNTO__
#include <sys/osinfo.h>
#endif

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

//////////////////////////////////////////////////////////////////////
//Added to provide support for help
/////////////////////////////////////////////////////////////////////
#include<photon/PxHelp.h>

/* Local headers */
 #include "abimport.h"
#include "proto.h"
#include "trekclock.h"

//Some storage for the numbers, bitmaps and stuff
PhImage_t *fnums[11];
PhImage_t *knums[11];
PtWidget_t *digits[6];
PhImage_t *ab[2];
PhImage_t *cl[2];
PhImage_t *alarm_but[2];
PhImage_t *cancel[2];
PhImage_t *set[2];
PhImage_t *alarm_array[2][2];
PtWidget_t *ship;

//A pointer to my database picture - symbols
ApDBase_t *mydb;

//Set up a global array of args to use
PtArg_t arg[5];
int clock_mode=STANDARD;
int time_digits[6];
int hour24=0;
int ALARM[2];
int picture_num=0;
char almsg[81];

void load_arrays(void);
void set_digits(void);
void load_values(void);

int
setup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	//////////////////////////////////////////////////////////////////////
	//Open the database picture which has been defined as an internal link
	//through PhAB.
	//////////////////////////////////////////////////////////////////////

	mydb=ApOpenDBase(ABM_symbols);

	/////////////////////////////////////////////////////////////////////
	//Just some setup stuff	
	//////////////////////////////////////////////////////////////////////
	load_arrays();
	set_digits();
	load_values();

	///////////////////////////////////////////////////////////////////////
	//	PxHelpTopicRoot("/Trek Clock Help");
	//Commented out because of a bug in the latest helpviewer calls.
	///////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////
	//Force these out of range so they'll never go off until the user
	//sets them.
	/////////////////////////////////////////////////////////////////////
	ALARM[0]=100;
	ALARM[2]=100;
	
	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );

	}

int alarm_proc(void)
	{
	//////////////////////////////////////////////////////////////////
	//This is the animation that is displayed when the alarm goes off.
	//Create a ship widget from the database and change it's position
	//resource to give the illusion of movement.
	//There is a better way of doing this but this works and you can see
	//what I'm doing.
	//I should have added a Timer and a work proc that would move
	//the widget every time the timer fired.
	//////////////////////////////////////////////////////////////////
	PhPoint_t ship_pos;
	int i,len,add;
#ifndef __QNXNTO__
	struct _osinfo os;


	/////////////////////////////////////////////////////////////////////
	//A method of determining what type of cpu we're running on and make an
	//adjustment to the increment value of the ship's x position. This is
	//to speed up the animation on slower cpu machines.
	//////////////////////////////////////////////////////////////////////
	qnx_osinfo(0,&os);
	if(os.cpu==586)
		add=2;
	else if(os.cpu==486)
		add=5;
	else if(os.cpu==386)
		add=10;
#else
	// use baud aware stuff
	add=5;
#endif
	
	ship_pos.y=60;

	/////////////////////////////////////////////////////////////////
	//Force the newly created widget to have these values set.
	////////////////////////////////////////////////////////////////
	PtSetArg(&arg[0],Pt_ARG_BORDER_WIDTH,0,0);
	PtSetArg(&arg[1],Pt_ARG_FILL_COLOR,Pg_BLACK,0);
	PtSetArg(&arg[2],Pt_ARG_MARGIN_HEIGHT,0,0);
	PtSetArg(&arg[3],Pt_ARG_MARGIN_WIDTH,0,0);
	if(clock_mode==STANDARD)
		{
		ship=ApCreateWidget(mydb,"fship",-214,60,4,arg);
		len=210;
		}
	else
		{
		ship=ApCreateWidget(mydb,"kship",-144,60,4,arg);
		len=140;
		}

	PtRealizeWidget(ship);

	//////////////////////////////////////////////////////////////////
	//Loop to animate the ship.
	///////////////////////////////////////////////////////////////////
	for(i=-len;i<477;i+=add)
		{
		ship_pos.x=i;
		PtSetArg(&arg[0],Pt_ARG_POS,&ship_pos,Pt_BLIT_FORCE);
		PtSetResources(ship,1,arg);
		if(i==370 || i==371)
			{
			PtSetArg(&arg[0],Pt_ARG_COLOR,Pg_WHITE,0);
			PtSetArg(&arg[1],Pt_ARG_TOP_BORDER_COLOR,Pg_WHITE,0);
			PtSetArg(&arg[2],Pt_ARG_BOT_BORDER_COLOR,Pg_WHITE,0);
			PtSetArg(&arg[3],Pt_ARG_TEXT_STRING,almsg,0);
			PtSetResources(ABW_alarm_printout,4,arg);
			}
		PtBkgdHandlerProcess();
		PtFlush();
		}

	ALARM[0]=100;
	ALARM[1]=100;
	return(Pt_END);
	}

void load_values(void)
	{
	/////////////////////////////////////////////////////////////////
	//Stuff the appropriate bitmap images into the label widgets
	//on the base window to give the impression of time ticking away.
	/////////////////////////////////////////////////////////////////
	int i;

	if(clock_mode==STANDARD)
		{
		for(i=0;i<6;i++)
			{
			PtSetArg(&arg[0],Pt_ARG_LABEL_DATA,fnums[time_digits[i]],
				sizeof(*fnums[time_digits[i]]));
			PtSetResources(digits[i],1,arg);
			}//for
		PtSetArg(&arg[1],Pt_ARG_LABEL_DATA,fnums[10],sizeof(*fnums[10]));
		}//standard
	else
		{
		for(i=0;i<6;i++)
			{
			PtSetArg(&arg[0],Pt_ARG_LABEL_DATA,knums[time_digits[i]],
				sizeof(*knums[time_digits[i]]));
			PtSetResources(digits[i],1,arg);
			}//for
		PtSetArg(&arg[1],Pt_ARG_LABEL_DATA,knums[10],sizeof(*knums[10]));
		}//klingon
	PtSetResources(ABW_colon,1,&arg[1]);
	PtSetResources(ABW_colon2,1,&arg[1]);
	}


void set_digits(void)
	{
	/////////////////////////////////////////////////////////////////////
	//Take the current time and load the time_digits array with the 
	//corresponding values for hours, minutes and seconds.
	////////////////////////////////////////////////////////////////////
	//What I need here is a tm structure with the current local time in it 
	//so that I can manipulate the elements in order to load the time_digits
	//array.
	////////////////////////////////////////////////////////////////////
	time_t time_of_day=time(NULL);
	register struct tm *ct=localtime(&time_of_day);
	
	///////////////////////////////////////////////////////////////////////	
	//Make some adjustments to the number of hours to allow for 12/24 hour
	//clocks.
	////////////////////////////////////////////////////////////////////////

	if(hour24 && ct->tm_hour > 12)
		ct->tm_hour-=12;
	if((!hour24) && ct->tm_hour==0)
		ct->tm_hour=12;		
	///////////////////////////////////////////////////////////////////////
	//Now the clever bit. Put the tens of hours, hours, tens of minutes,
	//minutes, etc into the time_digits array so that I can use them as
	//indexes into my PtImage_t arrays.
	/////////////////////////////////////////////////////////////////////

	time_digits[0]=(ct->tm_hour-(ct->tm_hour %10))/10;
	time_digits[1]=ct->tm_hour % 10;
	time_digits[2]=(ct->tm_min-(ct->tm_min % 10))/10;
	time_digits[3]=ct->tm_min % 10;
	time_digits[4]=(ct->tm_sec-(ct->tm_sec % 10))/10;
	time_digits[5]=ct->tm_sec % 10;

	if((ct->tm_min==ALARM[1]) && (ct->tm_hour==ALARM[0]))
		{
		PtAppAddWorkProc(NULL,(PtWorkProc_t)alarm_proc,NULL);
		ALARM[0]=100;
		ALARM[1]=100;
		}
	
	}


void load_arrays(void)
	{
	//////////////////////////////////////////////////////////////////////
	//Just stuff the values of the various number widgets from our database
	//picture into the PhImage_t global arrays to make accessing them a bit
	//easier and a lot faster.
	//////////////////////////////////////////////////////////////////////

	fnums[0]=ApGetImageRes(mydb,"fzero");
	fnums[1]=ApGetImageRes(mydb,"fone");
	fnums[2]=ApGetImageRes(mydb,"ftwo");
	fnums[3]=ApGetImageRes(mydb,"fthree");
	fnums[4]=ApGetImageRes(mydb,"ffour");
	fnums[5]=ApGetImageRes(mydb,"ffive");
	fnums[6]=ApGetImageRes(mydb,"fsix");
	fnums[7]=ApGetImageRes(mydb,"fseven");
	fnums[8]=ApGetImageRes(mydb,"feight");
	fnums[9]=ApGetImageRes(mydb,"fnine");
	fnums[10]=ApGetImageRes(mydb,"fcolon");

	knums[0]=ApGetImageRes(mydb,"kzero");
	knums[1]=ApGetImageRes(mydb,"kone");
	knums[2]=ApGetImageRes(mydb,"ktwo");
	knums[3]=ApGetImageRes(mydb,"kthree");
	knums[4]=ApGetImageRes(mydb,"kfour");
	knums[5]=ApGetImageRes(mydb,"kfive");
	knums[6]=ApGetImageRes(mydb,"ksix");
	knums[7]=ApGetImageRes(mydb,"kseven");
	knums[8]=ApGetImageRes(mydb,"keight");
	knums[9]=ApGetImageRes(mydb,"knine");
	knums[10]=ApGetImageRes(mydb,"kcolon");

	digits[0]=ABW_hour10;
	digits[1]=ABW_hour;
	digits[2]=ABW_minute10;
	digits[3]=ABW_minute;
	digits[4]=ABW_sec10;
	digits[5]=ABW_sec;

	cl[0]=ApGetImageRes(mydb,"fchange");
	cl[1]=ApGetImageRes(mydb,"kchange");
	
	ab[0]=ApGetImageRes(mydb,"fabout");
	ab[1]=ApGetImageRes(mydb,"kabout");

	alarm_but[0]=ApGetImageRes(mydb,"falarm");
	alarm_but[1]=ApGetImageRes(mydb,"kalarm");

	set[0]=ApGetImageRes(mydb,"fset");
	set[1]=ApGetImageRes(mydb,"kset");

	cancel[0]=ApGetImageRes(mydb,"fcancel");
	cancel[1]=ApGetImageRes(mydb,"kcancel");

	alarm_array[0][0]=ApGetImageRes(mydb,"ufplogo");
	alarm_array[0][1]=ApGetImageRes(mydb,"farmed");
	alarm_array[1][0]=ApGetImageRes(mydb,"klinglogo");
	alarm_array[1][1]=ApGetImageRes(mydb,"karmed");

	}
